#!/bin/bash
# $Id: flow-ga.sh,v 1.17 2014/12/08 20:47:11 stubbim Exp $
# $Source: /home/CVSROOT/netflow_ga/bin/flow-ga.sh,v $

# Dieses Skript wird von nfcapd aufgerufen und startet flow-ga.pl
# (getestet mit nfcapd version 1.6.12)

FLOW_FILE="$1"
SYSLOG_FACILITY=local5

case `uname` in
    "FreeBSD")
	PROC_RUNNING=`ps x| grep flow-ga.pl| egrep -v "grep|vi |vim|tail|capture" |wc -l`

	timeout() {
		time=$1

		# starte das Kommando in einer eigenen Shell um Probleme mit
		# Pipes zu umgehen
		command="/bin/bash -c \"$2\""

		expect -c "set echo \"-noecho\"; set timeout $time; spawn -noecho $command; expect timeout { exit 1 } eof { exit 0 }"    

		if [ $? -eq 1 ] ; then
			echo "Timeout after ${time} seconds"
		fi
	}

	;;
    "Linux")
		PROC_RUNNING=`ps -ef| grep flow-ga.pl| egrep -v "grep|vi |vim|tail|capture" |wc -l`
	;;
		*)
esac



# Skript beenden wenn eine andere Programminstanz bereits luft
if [ "$PROC_RUNNING" -gt 0 ] ; then
	logger -p $SYSLOG_FACILITY.error "another flow-ga process is running => exit"
	exit 1
fi

if [ -z $FLOW_FILE ]; then
	# Variable ist nicht gesetzt
	echo "missing flow file"

elif [ -f $FLOW_FILE ]; then
	# ermittle das Zeitintervall der Flow-Datei. Aus diesem Intervall wird
	# das Timeout fr das Perlskript  flow-ga.pl  berechnet.
	FLOW_START=`nfdump -I -r $FLOW_FILE | grep First | awk '{ print $2 }'`
	FLOW_END=`nfdump -I -r $FLOW_FILE | grep Last | awk '{ print $2 }'`
	FLOW_RUNTIME=`expr $FLOW_END - $FLOW_START`
	if [ $FLOW_RUNTIME -gt 300 ] ; then
		# maximales Timeout bei normalem Intervall von 5 Minuten
		FLOW_RUNTIME=295
	elif [ $FLOW_RUNTIME -gt 30 ] ; then
		let FLOW_RUNTIME-=6
	elif [ $FLOW_RUNTIME -lt 0 ] ; then
		# Fehlerbehebung wenn der Zeitstempel für Ende *vor* dem Anfang liegt
		# First:2147483647, Last:0
		FLOW_RUNTIME=2147483647
	fi
	logger -p $SYSLOG_FACILITY.debug "timeout $FLOW_RUNTIME sec, file $FLOW_FILE"

	# starte das Perlskript zum Berichten der Flows an Google Analytics
	timeout $FLOW_RUNTIME perl /usr/bin/flow-ga.pl $FLOW_FILE --debug --format nfdump

	# lokale Flowdatei lschen um das System schlank zu halten
	rm -f $FLOW_FILE

else
	logger -p $SYSLOG_FACILITY.error "flow file $FLOW_FILE does not exist"
fi
